package com.sunwin.metro.utils;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 字符串工具类
 * @author kane
 * @date 2019/12/4 17:52
 */
public class PerStringUtils {
    /**
     * 判断字符串是空或者是-1
     * @param str 字符串
     * @author kane
     * @date 2019-08-30 17:18
     **/
    public static boolean isNullOrEmptyOrNegativeOne(String str) {
        return isNullOrEmpty(str) || "-1".equals(str.trim());
    }

    /**
     * 判断字符串str是否为空或者等于str2
     * @param str  字符串
     * @param str2 字符串2，用于判断是否等于str
     * @author kane
     * @date 2019-11-12 9:24
     */
    public static boolean isNullOrEmptyOrEqual(String str, String str2) {
        return isNullOrEmpty(str) || str.trim().equals(str2);
    }

    /**
     * 判断字符串是否为 null或者为空字符串
     * @param str 字符串
     * @return boolean
     */
    public static boolean isNullOrEmpty(String str) {
        return str == null || str.trim().length() == 0;
    }

    /**
     * 判断object是否为空，或者转换成字符串（ String.valueOf(object)）是否为空字符串
     * @param object Object对象
     * @author kane
     * @date 2019-08-28 14:52
     **/
    public static boolean isNullOrEmpty(Object object) {
        return object == null || String.valueOf(object).trim().length() == 0;
    }

    /**
     * 判断集合是否是null值或没有元素
     * @param collection 继承collection的类
     * @return boolean
     */
    public static boolean isNullOrEmpty(Collection<?> collection) {
        return collection == null || collection.size() == 0;
    }

    /**
     * 传入的多个字符串是否包含null或空字符串
     * @param str1 字符串1
     * @param str2 字符串2
     * @author kane
     * @date 2019-09-05 16:19
     **/
    public static boolean containsNullOrEmpty(String str1, String str2) {
        return isNullOrEmpty(str1) || isNullOrEmpty(str2);
    }

    /**
     * 判断字符串数组中是否包含null或空字符串
     * @param strs 字符串数组
     * @author kane
     * @date 2019-09-05 16:20
     **/
    public static boolean containsNullOrEmpty(String... strs) {
        if (isNullOrEmpty(strs)) {
            return true;
        }
        for (String str : strs) {
            if (isNullOrEmpty(str)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断对象是否为null，以及转换成字符串是否为空字符串
     * @param objects 对象数组
     * @author kane
     * @date 2019-08-30 17:21
     **/
    public static boolean containsNullOrEmpty(Object... objects) {
        if (isNullOrEmpty(objects)) {
            return true;
        }
        for (Object str : objects) {
            if (isNullOrEmpty(str)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断对象是否为空
     * @param object 字符串
     * @author kane
     * @date 2019-09-05 16:20
     **/
    public static boolean isNull(Object object) {
        return object == null;
    }

    /**
     * 判断对象是都包含null
     * @param object1 对象1
     * @param object2 对象2
     * @author kane
     * @date 2019-09-05 16:28
     **/
    public static boolean containsNull(Object object1, Object object2) {
        return object1 == null || object2 == null;
    }

    /**
     * 判断对象是都包含null
     * @param object1 对象1
     * @param object2 对象2
     * @param object3 对象3
     * @author kane
     * @date 2019-09-05 16:30
     **/
    public static boolean containsNull(Object object1, Object object2, Object object3) {
        return object1 == null || object2 == null || object3 == null;
    }

    /**
     * 判断数组是否是null或者没有元素
     * @param array 数组
     * @author kane
     * @date 2019-09-05 16:33
     **/
    public static boolean isNullOrEmpty(Object[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 将字符串数组转换成以符号separator分割的字符串
     * @param values    数组
     * @param separator 分隔符
     * @return String
     */
    public static String join(String[] values, String separator) {
        StringBuilder result = new StringBuilder();
        if (values != null && values.length > 0) {
            for (Object value : values) {
                result.append(value).append(separator);
            }
            result = new StringBuilder(result.substring(0, result.length() - 1));
        }
        return result.toString();
    }

    /**
     * 将各种类型的对象数组或者集合转换成字符串数组
     * @param object 对象
     * @author kane
     * @date 2019-09-05 17:16
     **/
    public static String[] toStringArray(Object object) {
        if (object == null) {
            return null;
        }
        String[] strArray = null;
        // 如果为对象数组
        if (object instanceof Object[]) {
            Object[] objArray = (Object[]) object;
            if (objArray.length > 0) {
                strArray = new String[objArray.length];
                for (int i = 0; i < objArray.length; i++) {
                    strArray[i] = String.valueOf(objArray[i]);
                }
            }
        } else if (object instanceof List) {
            // 如果为集合
            List<?> list = (List<?>) object;
            if (list.size() > 0) {
                strArray = new String[list.size()];
                for (int i = 0; i < list.size(); i++) {
                    strArray[i] = String.valueOf(list.get(i));
                }
            }
        }
        return strArray;
    }

    /**
     * 将符号separator分割的字符串转换成 List&lt;String&gt;
     * @param str       字符串
     * @param separator 分隔符
     */
    public static List<String> stringToList(String str, String separator) {
        if (str != null && str.trim().length() > 0) {
            return new ArrayList<>(Arrays.asList(str.split(separator)));
        }
        return Collections.emptyList();
    }

    /**
     * 查找字符串中关于某个组的索引
     * @param regex 正则表达式
     * @param input 给定输入的字符串
     * @param group 组号
     * @return 组所在字符串的索引
     */
    public static int getIndex(String regex, String input, int group) {
        int index = -1;
        try {
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(input);
            if (m.matches()) {
                index = m.start(group);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        stringToList("", "");
        return index;
    }

    /**
     * 按逗号分割字符串
     * @param str 字符串
     * @return 字符串数组
     * @author kane
     * @date 2017-3-29 上午9:53:12
     */
    public static String[] splitStrByComma(String str) {
        if (str == null || str.length() == 0) {
            return new String[]{};
        } else {
            return str.split("\\s*,\\s*|\\s+");
        }
    }

    /**
     * 按逗号分割字符串并转换成set集合
     * @param str 待分割的字符串
     * @return Set<String>
     * @author kane
     * @date 2017-3-29 上午9:58:44
     */
    public static Set<Integer> splitStrByCommaToSet(String str) {
        String[] strArray = splitStrByComma(str);
        Set<Integer> set = new HashSet<>();
        for (String string : strArray) {

            if (string.trim().length() == 0) {
                continue;
            }
            set.add(Integer.valueOf(string));

        }
        return set;
    }

    /**
     * 对url进行解码
     * @param encodeUrl 编码好的url
     * @return 返回解码后的url
     * @author kane
     * @date 2017-6-19 下午2:07:10
     */
    public static String urlDecoder(String encodeUrl) {
        if (isNullOrEmpty(encodeUrl)) {
            return encodeUrl;
        }
        try {
            return java.net.URLDecoder.decode(encodeUrl, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return encodeUrl;
    }

    /**
     * 对url进行编码
     * @param url 待编码的url
     * @return 返回编码好的url
     * @author kane
     * @date 2017-6-19 下午2:07:42
     */
    public static String urlEncoder(String url) {
        if (isNullOrEmpty(url)) {
            return url;
        }
        try {
            return java.net.URLEncoder.encode(url, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return url;
    }

    /**
     * 替换空格、tab键、换行符为空格（\\s*|\t|\r|\n）
     * @param sourceStr 源字符串
     * @return 替换后的字符串
     * @author kane
     * @date 2017-6-21 下午3:16:25
     */
    public static String replaceCRLF(String sourceStr) {
        String replaceSource = null;
        if (!isNullOrEmpty(sourceStr)) {
            Pattern p = Pattern.compile("\\s*|\t|\r|\n");
            Matcher m = p.matcher(sourceStr);
            replaceSource = m.replaceAll("");
        }
        return replaceSource;
    }

    /**
     * 如果字符串为null返回空字符串
     * @param str 字符串
     * @return String
     * @author kane
     * @date 2020-04-17 17:14
     */
    public static String defaultString(String str) {
        return defaultString(str, "");
    }

    /**
     * 如果字符串为null返回默认字符串
     * @param str 字符串
     * @return String
     * @author kane
     * @date 2020-04-17 17:14
     */
    public static String defaultString(String str, String defaultString) {
        return str == null ? defaultString : str;
    }

    /**
     * 如果字符串为空或-1则返回默认值
     * @param str        字符串
     * @param defaultStr 默认值
     * @return String
     * @author kane
     * @date 2020-04-29 16:31
     */
    public static String defaultIfEmptyOrNegativeOne(String str, String defaultStr) {
        return isNullOrEmpty(str) || "-1".equals(str) ? defaultStr : str;
    }

    /**
     * 判断文本中是都包含对应的词
     * @param text  文本
     * @param words 词集合
     * @return boolean
     * @author kane
     * @date 2020-06-19 11:07
     */
    public static boolean contains(String text, String... words) {
        if (PerStringUtils.isNullOrEmpty(text) || PerStringUtils.isNullOrEmpty(words)) {
            return false;
        }
        for (String word : words) {
            if (text.contains(word)) {
                return true;
            }
        }
        return false;
    }


    /**
     * 判断字符串是否是乱码
     * @param text 字符串
     * @return 是否是乱码
     */
    public static boolean isChineseMessyCode(String text) {
        return isChineseMessyCode(text, 0.4);
    }

    /**
     * 判断字符串是否是乱码
     * 表情处理参考下面
     * https://unicode.org/Public/emoji/13.0/emoji-sequences.txt
     * http://unicode.org/emoji/charts/full-emoji-list.html
     * https://www.cnblogs.com/sosoft/p/3456631.html
     * https://github.com/zly394/EmojiRegex
     * https://www.cnblogs.com/yuanke/p/7889899.html
     * @param text      字符串
     * @param threshold 判断是乱码的域值
     * @return 是否是乱码
     */
    public static boolean isChineseMessyCode(String text, double threshold) {
        // 判断是不是gbk编码（完全兼容GB2312-80标准，支持国际标准ISO/IEC10646-1和国家标准GB13000-1中的全部中日韩汉字，并包含了BIG5编码中的所有汉字），不是则认为可能是乱码
        if (text == null || Charset.forName("gbk").newEncoder().canEncode(text)) {
            return false;
        }
        // 去掉 tab键、回车换行等计算百分比
        text = text.replaceAll("\\s*|\t*|\r*|\n*", "");
        text = text.replaceAll("\\p{P}", "");
        // 去除表情
        text = text.replaceAll("(?:[\uD83C\uDF00-\uD83D\uDDFF]|[\uD83E\uDD00-\uD83E\uDDFF]|[\uD83D\uDE00-\uD83D\uDE4F]|[\uD83D\uDE80-\uD83D\uDEFF]|[\u2600-\u26FF]\uFE0F?|[\u2700-\u27BF]\uFE0F?|\u24C2\uFE0F?|[\uD83C\uDDE6-\uD83C\uDDFF]{1,2}|[\uD83C\uDD70\uD83C\uDD71\uD83C\uDD7E\uD83C\uDD7F\uD83C\uDD8E\uD83C\uDD91-\uD83C\uDD9A]\uFE0F?|[\u0023\u002A\u0030-\u0039]\uFE0F?\u20E3|[\u2194-\u2199\u21A9-\u21AA]\uFE0F?|[\u2B05-\u2B07\u2B1B\u2B1C\u2B50\u2B55]\uFE0F?|[\u2934\u2935]\uFE0F?|[\u3030\u303D]\uFE0F?|[\u3297\u3299]\uFE0F?|[\uD83C\uDE01\uD83C\uDE02\uD83C\uDE1A\uD83C\uDE2F\uD83C\uDE32-\uD83C\uDE3A\uD83C\uDE50\uD83C\uDE51]\uFE0F?|[\u203C\u2049]\uFE0F?|[\u25AA\u25AB\u25B6\u25C0\u25FB-\u25FE]\uFE0F?|[\u00A9\u00AE]\uFE0F?|[\u2122\u2139]\uFE0F?|\uD83C\uDC04\uFE0F?|\uD83C\uDCCF\uFE0F?|[\u231A\u231B\u2328\u23CF\u23E9-\u23F3\u23F8-\u23FA]\uFE0F?)", "");
        //TODO:此处需要注意
        text = text.replaceAll("(?:[\u0080\u00a0\u00A2\u00A3\u00A5\u00AA\u00AF\u00B0-\u00BE\u00D7\u00F7]"  + "\\u005e" + "\u005f" + "]|[\u0020-\u002F" + "\u003A" + "-\u003F\u0060\u007B" + "-\u007E]|[\u2150" + "-\u218F\u2200" + "-\u22FF\u2300-\u23DB\u23E0-\u23FF\u2400-\u242F\u2440-\u244F\u2460-\u24FF\u27C0-\u27EF\u2980-\u29FF\u2A00-\u2AFF]|[\u2010-\u2027\u2030-\u205E\u2E00-\u2E4F\u2070-\u209F\u00AE\u00A9\u0099\u20A0-\u20BF\u2190-\u21FF\u27F0-\u27FF\u2900-\u297F\u2500-\u257F\u2580-\u259F\u25A0-\u25FF\u2600-\u26FF\u2700-\u27FF\u2B00-\u2BFF]?)", "");
        //去除表情方法
        //text = EmojiParser.removeAllEmojis(text);
        // 去除私有
        text = text.replaceAll("\uE000-\uF8FF", "");
        char[] ch = text.trim().toCharArray();
        float chLength = ch.length;
        float count = 0;
        for (char c : ch) {
            if (!Character.isLetterOrDigit(c)) {
                if (!isChinese(c)) {
                    count = count + 1;
                }
            }
        }
        double result = (count / (1.0 * chLength));
        // 超过threshold算乱码
        return result > threshold;
    }

    /**
     * 判断字符是否是中文
     * @param c 字符
     * @return 是否是中文
     */
    public static boolean isChinese(char c) {
        Character.UnicodeBlock ub = Character.UnicodeBlock.of(c);
        // @formatter:off
        return ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS
                || ub == Character.UnicodeBlock.GENERAL_PUNCTUATION
                || ub == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION
                || ub == Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_FORMS
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY
                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT
                || ub == Character.UnicodeBlock.CJK_STROKES
                || ub == Character.UnicodeBlock.ENCLOSED_CJK_LETTERS_AND_MONTHS
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_C
                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D
                ;
        // @formatter:on
    }

    public static void main(String[] args) {
        String text1 = "145";
        System.out.println(text1.replaceAll("[1-35-7]", "x"));
        String text = "⽬×!\"#$%&'()*+,-./:;<=>?@[\\]^_{|}~ :;<=>?ⅾtitle不过节，只吸猫 (? ??? ??? ?), text:( )o(= =)m(=^･ ･^=)丿~o( = = )m在吸猫这件事上，东瀛人从来都是当仁不让的第一名。猫咖啡馆、猫咪书店、猫岛，无处不在的“吸猫”场所足以彰显他们对猫的狂热，人气超高的哆啦A梦、Hello Kitty更以猫为载体，让二次元文化风靡全球，享誉世界。电影《猫的报恩》而在日本成千上万的“猫奴”中，最有个性的莫过于那些文豪。夏目漱石、宫泽贤治、萩原朔太郎 这些大名鼎鼎的作家，无一例外都是写猫的高手。对于他们来说，猫不仅是书案边的老友、庭院中的精灵，更是难以替代的灵感源泉和心灵寄托。文豪吸猫，与普通人有什么不同？他们是如何将那些关于猫的回忆具象于文字，又是如何透过“猫眼”审视着这个世界？夏目漱石：养猫千日，写猫一时夏目漱石可以说是日本“猫文学”的开创者，与此同时，他也是一位资深的“铲屎官”。他曾自述养过三只猫，其中成为《我是猫》中主人公原型的便是他养的第一只猫。那是一只迷路的野猫，偶然闯入漱石家，几次被家人驱赶出去，又总是跑回来，令他十分头痛，只得养在家中。起初，夏目漱石并没有对猫表现出过多的喜爱，也没有给猫取名，只是叫它为“猫”。但渐渐地，他开始关心这只无名猫的一举一动。某一年新年，漱石的妻子无意间发现猫正在厨房里偷吃孩子们吃剩下的杂煮年糕，不断抬起两只前爪手舞足蹈地拼命扑腾。女佣们也看到了这一幕，觉得这猫实在是太贪吃，都大笑起来。漱石听说后，将这件滑稽小事写进了《我是猫》里：“咱家一边盯住碗底一边想：假如有人来才好呢。可是，终于没人来，也就终于非吃年糕不可了。于是，咱家将全身重量压向碗底，将年糕的一角叼住一寸多长。使出这么大的力气叼住，按理说，差不多的东西都会被咬断。然而，我大吃一惊。当我以为已经咬断而将要拔出牙来时，却拔也拔不动。本想再咬一下，可牙齿又动弹不得。当我意识到这年糕原来是个妖怪时，已经迟了。”随着《我是猫》的大受好评，这只在书中极富个性的无名猫也成了夏目漱石家的“福猫”，却依旧难改本色，常常因为太淘气让主人大伤脑筋。猫很爱恶作剧，经常在半夜钻进已经熟睡的家人的被窝中又抓又挠，气得夏目漱石抓起身边的尺子，将其教训一番。虽然嘴上总是埋怨，但在猫死后，夏目漱石还是给猫造坟，并在墓碑上题词“九泉之下再无电闪雷鸣”，甚至将猫的讣告寄给弟子们：“家猫沉疴难返，药石无效，于昨夜某时逝于杂物间炉灶之上，今天托车夫装箱，于后院中举行了葬仪。”最后，他又补上了一句：“由于主人目前正在执笔《三四郎》中，故不用特地前来参加葬礼。”这段细碎却十分温馨的故事，被他记录在了《猫之墓》这则小文中。宫泽贤治：我不爱猫，猫却最懂我同样是家喻户晓的文学大师，《银河铁道之夜》的作者宫泽贤治对猫的感情显然复杂得多。在早年所写的一篇名为《猫》的散文中，宫泽贤治曾说 “我很讨厌猫”，可在其他作品中，猫却屡次作为主角或是重要的角色登场。比如童话在《猫的事务所》中，他构建了一个如人类社会一般的猫世界，描绘出一幅“社畜猫咪”众生相。而《橡子与山猫》中那只身披金黄色斗篷，威风凛凛地对着橡子们宣判的大山猫令人印象深刻，甚至成了宫崎骏创作出动画《龙猫》的灵感源泉。山猫呆呆地站着，脚边橡果吱吱作响，让我印象深刻。——宫崎骏除了这两个故事之外，宫泽贤治还在《大提琴手高修》《蜘蛛、鼻涕虫和狸猫》《规矩特别多的饭店》等几个故事中都写到了猫。在这些故事中，猫咪们一改可爱画风，有的态度傲慢、装腔作势，有的胆小懦弱、自怨自艾，但无一例外的是，从它们身上处处能看到人类的影子。宫泽贤治借着描绘与人类性格相似的猫，将社会中的种种现象立足于猫界，进行辛辣的讽刺。另外，对热爱自然的宫泽贤治来说，猫也是人类破坏自然界的受害者，通过写猫的故事对虚伪傲慢的人类进行批判，或许也是他虽并非“猫奴”，但从不停止写猫的理由。除此之外，他人笔下的宫泽贤治也与猫有着不解之缘。1983年，漫画家增村博在将宫泽贤治的代表作《银河铁路之夜》改编为漫画时，选择将故事的两个主人公的形象设计成了猫；1996年，在河森正治导演的宫泽贤治传记动画电影《伊哈托布幻想——贤治之春》中，宫泽贤治和其他人也都以猫的形象出现。也许在日本人眼中，猫已经成为宫泽贤治的代名词了吧。《银河铁道之夜》动画版海报萩原朔太郎：转角，遇到猫说起宫崎骏笔下不可思议的“猫故事”，除了《龙猫》之外，可能会有不少人想起吉卜力工作室的另一部动画《猫的报恩》。而与之相似的故事情节，早在被誉为“日本近代诗之父”的诗人萩原朔太郎的散文诗《猫町》中就已经出现。诗人在散步途中迷失了方向，无意间走入一座陌生的城镇。城中景色无比精巧繁华，却在一瞬之间生出异象：“猫、猫、猫、猫、猫、猫、猫。猫的身影随处可见。留着胡须的猫脸，从家家户户的窗口中探出来，就像裱在框里的画一样，占满了所有空间。”幻化成人形的猫群鱼贯而出，仿佛要淹没整座城镇。而就在下一瞬间，那些猫又突然从街上消失，仿佛不曾存在过一般，小镇也恢复了原先门可罗雀、平平无奇的模样。吉卜力工作室《猫的报恩》如果说作家与猫一样，都拥有细腻的情感与自由的灵魂，那么萩原朔太郎无疑是最像猫的诗人。他的文字本身如猫一般，带着与生俱来的优雅，又充满柔和轻快的韵律感，只要用心去聆听，仿佛就能感知不远处猫咪的轻声低语。不同于其他作家写猫，或可爱顽皮，或高冷傲慢，在萩原朔太郎的笔下，猫似乎不再拥有实体，而是成了一种象征着神秘而独立的力量，驱使他摆脱一成不变的景象与枯燥乏味的生活，驰骋于精神世界，仿佛那不曾为人知晓的另一个次元，正缓缓向他开启大门。猫精灵居住的城镇，确实存在于宇宙的某处，一定存在着。——萩原朔太郎究竟是诗人误闯神秘莫测的猫城，还是在梦中幻化为猫，从而察觉到了那些被世人忽略的奇妙景色，我们已经无从知晓。就好像庄周梦蝶，是梦是醒，恐怕连本人也难以分清。如今，传说中《猫町》的故事发源地——东京的下北泽，早已成为众多读者与 “猫奴”的巡礼圣地。也许有一天，他们也会追随着诗人的脚步，在某个恍惚的瞬间，穿过陌生的街角，推开那扇连接着人界与猫界的神秘之门。在猫面前，那些痴迷于猫的文豪也如所有普通人一般，将所有的喜怒哀乐投射于这狡黠的小动物身上，感受着人类与猫之间的爱恨纠葛。而当他们将与猫相伴的琐碎往事记录于笔下，描绘猫生与人生的奇妙巧合，或将脑海中猫的意象点缀在字里行间，从猫的角度打量世界时，便会发觉那可爱身影的背后，隐藏着饱含人情冷暖、世间百态的另一面。你与猫之间，又有怎样一段独一无二的故事呢？《猫的九故事》暖萌上市，猫的眼中不只有天空，还有故事。《猫的九故事》 [日]夏目漱石等著 文绘 译从夏目漱石到宫泽贤治，从温馨小事到怪谈传奇，九位文学大师写下他们的猫故事，串联起无数遥想的时空，更是电影《龙猫》的灵感来源。更有新锐画师手绘53幅精美插图，均为国际插画大赛获奖作，在文字间隙捕捉猫儿的迷人之处，带来难以忘怀的温暖与感动。猫儿的故事，读这一本就够了！\n";
        text = "title2020/9/2�惧勾浜哄�垮��寰藉�����镐�璐归�ㄥ�灞�楂�绔�瀹㈡�锋���辨椿��, text:�ユ�锛�灞辫タ缁�娴�缃� 浣���锛� ��甯��堕�达�2020/9/2 17:33:09  ����瀹㈡�蜂互���♀��涓��存���惧勾浜哄�跨�����″����锛�涓烘�村ソ�版���″����棣���瀹㈡�峰�圭�惧勾浜哄�跨������涓����憋���2020骞村�㈡�����ヤ复涔���锛�瀹���淇�璐归�ㄧ�规��涓虹�辫�卞コ澹�涓惧��浜�涓��猴�濂充汉濡��扁����楂�绔�瀹㈡�锋���辨椿�ㄣ��  8��16�ヤ���9��30��锛�娲诲�ㄦ�ｅ�寮�濮�锛���浠����板�哄��瀹句�缁�浜��惧勾��VIP瀹㈡�锋���″���ㄧ��崇�辩��璧����＄��锛��ヤ��ョ�辨���辫��甯�璇�缁���涓哄ぇ瀹惰�茶В浜����辩��娴�绋����稿�虫��宸э�骞朵负澶у�跺���炬���卞伐�峰���辨��锛�杩�绋�涓�瀵规��浣���瀹捐�琛�涓�瀵逛�����瀵笺�����变�涓轰腑��浼�缁�����涓����风�硅�茬��涓��ㄨ�烘��锛�瀹�涓�浠���绠�����灏��辨��缁���������锛�瀹��存��涓�绉�瀵圭��娲荤����搴�锛�姣�涓�浣��ュ�哄��瀹惧�规���遍�芥������宸辩���圭��瑙�瑙ｏ�瀵圭��娲婚�芥��涓�绉����圭��灞�绀恒����娆℃椿�ㄤ��ㄤ�涓�涓�绮剧����变�����瀹���涓�锛���婊¤�藉���  ���卞��浠ラ�跺�舵�ф������楂�绮剧�����绱��讳��烘��淇��伙�澶у�堕�界悍绾锋��璋㈠���告��渚�杩��风��骞冲�帮����ワ���浠�灏�浼�涓惧���村�杩��风��娲诲��锛�灏�����瀹㈡�蜂互���♀����浣垮�借返琛��板���\n";
//        text="⫀ title不过";
//        text="〓 ！";
        text = "唳氞�靚璉�黻��唳灌�敿璉�n唳��唳��鶧瘥ZΖ唰囙Χ唰囙Π 唳涏�n唳傕Χ唳��唳多�i唳班�� 唳忇�{唳熰�o 唳ㄠ�H 唳��ㄟ瘥Z�黻��唳��敿瘥Z�n唳むΘ 唳��黻�曕�n唳班�o 唳班�H唰囙��唰囙イ 唳��黻璁h�o唳班Ν唳距�} 唳膏Ξ唰熰�m 唳��黻�曕�n唳班�o 唳側�n唳�_�黻��唳��敿��唳︵�黻�栢�� 唳ㄠ�o啷��  唳��黻�曕�n唳班�o唳��唳��n唳側�o唳��唳忇�{唳溹Θ 唳溹�n唳班�鶧聧�n唳��唳��n唳ㄠ�敿碻Tイ 唳む�o唳ㄠ�o 唳�_�n唳側�� 唳��鶧膣~�n唳ㄠ�鶧獢_�n唳班�o唳��唳�壓U唳む�� 唳��n唳班�黻��唳忇Μ唳��唳氞�靚璉�n 唳膏�n唳囙Θ 唳�_�n唳糞�n 唳溹�n唳ㄠ�黻璉�イ  唳��黻�曕�n唳��#2495;টি ছাংশা একটি শান্ত সড়কে অবস্থিত। বেকারির কাছ দিয়ে গেলে ব্রেড ও কফির সুগন্ধ পাওয়া যায়। সুগন্ধ ছাড়াও বেকারিতে ভালবাসা, পরিশ্রম ও উষ্ণতা অনুভব করা যায়।  'আমার বেকারির মুনাফা বেশি না, তবে খুবই আন্তরিকতাপূর্ণ।' জার্মান মালিক এমনটি বলেন।  তাঁর বেকারির আয়তন বড় না, কর্মীও বেশি না, বেশিরভাগ গ্রাহক হলেন পুরানো গ্রাহক।  মালিকের চীনা নাম উ চেং রং। তাঁর বয়স ৫০ বছরের বেশি। তিনি ভালোভাবে চীনা ভাষা বলতে পারেন। তিনি হুনান প্রদেশের আঞ্চলিক ভাষাও কিছু বলতে পারেন। কোন কোন গ্রাহক তাঁকে ডাকেন আঙ্কেল উ।  বেকারিটির রান্নাঘরে কাজের শব্দ ছাড়া অন্য কোনও শব্দ পাওয়া যায় না। সবাই শুধু কাজ করে।  উ চেং রং সবসময় ওভেনের কাছে দাঁড়িয়ে থাকেন। যখন ওভেন টাইমারের শব্দ শুনতে পান, তখন তিনি বাষ্পের রুটি নিজেই বের করে আনেন। উ চাচা বলেন, আমার সব কর্মী হলেন প্রতিবন্ধী মানুষ। তাঁরা টাইমারের শব্দ শুনতে পান না। সেজন্য আমি সবসময় ওভেনের কাছে দাঁড়িয়ে থাকি। যদিও আমার বেকারিতে শব্দ কম, তবুও এখানে উষ্ণতা অনেক বেশি।  ১৮ বছরের আগে উ চেং রং ও তাঁর স্ত্রী জার্মানি থেকে ছাংশা শহরে এসে শ্রবণ প্রতিবন্ধী শিশুদের সহায়তা দেওয়া শুরু করেন। সে সময় ছাংশা শহরে জার্মান একটি বেসরকারি দাতব্য সংস্থার উদ্যোগে শ্রবণ প্রতিবন্ধী শিশুদেরকে সহায়ক পরিকল্পনায় কাজ করতেন। তখন তাঁরা ভাবেন নি যে, ছাংশা শহরে ১৮ বছর থাকবেন!  ১৮ বছরে তিনি ও তাঁর স্ত্রী প্রায় ৫ শতাধিক শিশুকে সাহায্য করেছেন। তাঁরা শিশুদেরকে কথা বলা শিখিয়েছেন, শিশুদেরকে বিদ্যালয়ে লেখাপড়ায় সহায়তা করেছেন। উ চেং রং বলেন, ছাংশা শহরে খুবই গরম, প্রথমে এখানে কিছুটা অস্বস্তি লাগতো। আমার মনে হতো, আমাদের কাজ খুবই গুরুত্বপূর্ণ। আমরা সময় হিসাব করি নি, ধীরে ধীরে ১৮ বছর হয়ে গেছে!  ২০১১ সালে উ চেং রং প্রতিবন্ধী মানুষদের সহায়তা করার জন্য একটি বেকারি খোলার চিন্তা করেন। তিনি বলেন, যদি প্রতিবন্ধী মানুষ এ ধরনের প্রযুক্তি পায়, তাহলে তাঁদের জীবনযাত্রা উন্নত হবে। পরিশ্রম করে সম্মান অর্জন করতে পারবে।  কয়েক সপ্তাহ পর উ চেং রং একজন জার্মান প্যাস্ট্রি শেফকে নিয়ে আসেন। শেফ উ চেং রং ও তাঁর কর্মীদের বেকারি প্রযুক্তি শিখান।  প্রথমে বেকারিটি ছিল ব্যস্ত রাস্তায়। কিন্তু বেকারির মুনাফা বেশি হতো না। তাই তিনি বেকারিটি একটি শান্ত ও ছোট রাস্তায় স্থানান্তরিত করেন।  ২০১৪ সালে জার্মান শেফ জার্মানিতে ফিরে যান। এরপর তিনি নিজেই বধির কর্মীদেরকে কাজ শেখান। এ পর্যন্ত তিনি ২০জন প্রতিবন্ধী মানুষকে প্যাস্ট্রি তৈরির কাজ শিখিয়েছেন।  'আমি কখনও এদের ছেড়ে যাবার চিন্তা করি নি।'  উ চেং রং এভাবেই সাংবাদিককে জানান। এ সমস্ত বছর বেকারির কোন মুনাফা হয় নি। কিন্তু উ চেং রং তাদের ছেড়ে যান নি।  তিনি বলেন, ছাংশা শহরে তার কোনও নিজস্ব বড়ি নেই, ঘর ভারা করতে থাকেন। দেশ ত্যাগের এতগুলো বছর পরও তিনি মাতৃভূমি মিস করেন।  তিনি বলেন, আমি ছাংশা শহরে অনেক বন্ধু পেয়েছি। অধিকাংশই হলেন তাঁর বেকারির গ্রাহক।  'আমার স্ত্রীকে আমার পাশে থাকার জন্য ধন্যবাদ জানাই। তিনি থাকলে আমার একাকী লাগে না।' উ চেং রংয়ের জার্মান বন্ধু ও আত্মীয়রাও তাঁকে সমর্থন করেন। যখন বেকারির সময় কঠিন হয়, তাঁরা উ চেং রংকে সহায়তা করেন।  এ বছরে মহামারীর অবস্থায় বেকারি ক্ষতির মুখে পড়ে। কিন্তু উ চেং রং কোনও কর্মী ছাঁটাই করেন নি, তাঁর ব্রেডের দামও বাড়ান নি। কারণ তিনি ভাবছেন, তাঁর প্রতিবন্ধী কর্মীরা অন্য কাজ খুঁজে নিতে পারবে না।  এতগুলো বছরেও আমি তাদের ছেড়ে দেই নি, মহামারীর সময়ও আমি তাদের ছেড়ে দিতে পারি না।  জার্মান চাচা বেকারির দেয়ালে একটি স্লোগান লিখে রেখেছেন। লেখা হয়, ব্রেড কিনলে ভালোবাসা পাঠানো হবে। এ স্লোগান তাঁর অতিথিদেরকে দিয়েছেন। এ বেকারিতে তৈরি ব্রেড শুধু খাবারই নয়, বরং প্রতিবন্ধীদের তৈরি ভালোবাসা ও প্রেম!  এক পিস ব্রেড, এক কাপ কফি, এক দল মানুষ। যখন আপনি এ নীরব এ বেকারিতে প্রবেশ করবেন, তখন সে উষ্ণতা অনুভব করতে পারবেন।  (ছাই/তৌহিদ)";
        text = "����\\\\s��}�\\u0015���\\u001f�͛7\u007F��ŋ\\u0017\u007F���K�.���~�駟\u007F��[jԨQ�f͵�Z�nݺ���kРA�fͶ�j��W_=]��\\u0017\\u0002\\u0005D@�}\\u0001�TW�\\u00002=�~��7���\\u000b��b�5�X���Pd�\u007F�ᇷ�~{��\\u0001S�N\\u0005�N�:\\u0015y�r����\u007F�3g���?��/���{�\\u001d�Q\\u000f�|�-[��\\u0007\\u001f�6�\\u001a7nܪU�=��c��[y��%\\u0015�@a\\u0011�d_X�\\b\\u001d\\u0000\\u001f��v\\u001a�m���C\\u000fe�Y�s\\u0016Y匀�}9��h�\\u001b�}��\\u0007;=2�d��X~���H���~���-�}\\u0015��s�=w�]w���k6ӭ��\\u001adڴi\\u0003��V4G1��a����\\u0006�Νk\\u001d��  ��j\\u0012�@�\\u0019X�˜L���Ǐ����q��\\u0018\u007F������`��r0�3�\\u0001*�\\u000f\\u00011����f�\\u0005\\u0002�d`Y\\u0000�5M V�������H�1��c�=��\\u0013W[m5\\u000f��\\u0016\u007F��[n����o��c¿����YR\\\\�cݻRd�zy59! \\u0004rE��\\u000f?�֭\\u001b��aÆcƌ��^��\\u001e�\\u0019\\u000eE�رc!\\u00002 \\u0006�r���K:\\u0002b�I\u007F\\u00024\u007F! \\u0004�\\b�\\u001b\u07FD{��\\u000b\\u0017\\\"R�y����m*���!\\u0000�\u007F��$��Tθ\\u001a%f\\b���lA5\\u001d! \\u0004�E��7�8��S�\u007Fw���\\u000f\\u001e\\u0019\\u001a�C������ � ���WK���~\\u0010�)y)\u007FS�I���B\\u0004$�'n�G.0Dd$\\u0018fn�  �� \\u0005\\u0019/xl�'�`�\\u0002�B�\\u0016�}�\\u0019\\\\PPQ�\\u0019�H�M�e�x��%\\u0007\\u00011��u�fʋ�\\u001c�|��rP\\u001e\\\\�le\\u0004#�\\u000f\\u0004I�8~\\u0017Y��?֛i�y_\\u001b���ଢ଼�I\\u001b~ùA��\\u0018=��\\u0006�pw?\\u00130���W_�۰��� \\u0006�\\u0010>�\\u0010粫�� �*dr>�E*GYz�z�B���]\\u0005\\u0019�H�\\u0013~��͙�3P��g��  ����n\\u0019���@�K;�N�G\\u0007��o�u�]�Bǡ\\u001a�\\u0011��1\\u0006�8F\\u0012\\bb��\\u0003�\\u0011�o\\u000e��\\u0006�����8\\\\�\\u001e�=I�1�h\u007F8ʁ�W�� a$uv~y����\\u0010.o�\\\"m\\u0007��QA\\b8\\u0004��\\u001d\\u0014*dB��4#��2�R������\\u000f����\\u001bE\\u001c2F�T\u07FE\\u000e���z����  ]�t�����Z�SN��ǵ�]*v�\\u0019&�=P����D������\\u0010\\u0010\\u0002B��\\b�D�\\u001dS�={��o��O>�h\\u000e\\u001b6�\\u001a�����L_�n]�~���&�c�G�\u007F�G\\u001a\\u0010�|�\\u0015���C��\\u0004���k�\\u001e�z��G���K\\u0004�YK�\\u0015\\u0002B@\\b\\b\\u0001!�(\\u0004JB����_\\u0007\\u000e\\u001cx�I'ժU\\u000b~\u007F�)� �/[��\\u001f~8��Sm=\\u0016,X�*~\u07BCy������;��\\u000e?�p�~�p�v�~���[o�u�嗟w�y�?@�'l\\u000f�\u007F��l�2Q\\u000b�� \\u0001! \\u0004��\\u0010( fOR��W_��#�`=V^ye\\u0018�U$\\\"\\u001f3? |ڻK������O?��\u07BD{�-�V�����\\u0010\\u0010\\u0002B@\\b\\b�x#P��~\u0084 �>�������N�:uʔ)��A.'�\\u001eV��[§w�qG$���\\u0007� ��'��+��\\u0007��g\\u0007�g�M7� �??�{\\u001c�\\u0002+�ɿf͚�\\u0010���p� ���*\\u0004��\\u0010\\u0010\\u0002B �\\bT��t��g�YEr:[�h\\u0011|\\u001aK�q�\\u001d�@H�{\uEE671r��C\\u000f=��=F�\\nƋ�� \\u0001! \\u0004��\\u0010�\\u000b\\u00011{=\\u0007B@\\b\\b\\u0001! \\u0004b���}�\\u0017X�\\u0013\\u0002B@\\b\\b\\u0001! f�g@\\b\\b\\u0001! \\u0004�@�\\u0011\\u0010���\\u0002kzB@\\b\\b\\u0001! \\u0004�>���@\\bT-\\u0002�f�8q��ɓ�\u007F�}23V-1%8:�N4jԨ}����\\\\i���\\u0007z*2?��|*Rb��?���t�\\u0017/�P��K�~��w?��\\u0013I�h_�F 2�r>\\u000b'�իW�A�\\u0006͚5�j���\\u0010����U�PR��M�2{�k�\u038Bwz";
        text = FileUtils.getFileContent("C:\\Users\\hui\\Downloads\\161C7E17-383D-41fc-9E3C-E961B4C94662.txt", "utf-8");
        System.out.println(convertUnicode(text));
        System.out.println(text.replaceAll("[\\u2A00-\\u2AFF]", "c"));
        System.out.println("乱码：" + isChineseMessyCode(text));
        System.out.println(convertUnicode(" "));

        System.out.println(convertUnicode(text).contains("ffff"));

    }

    /**
     * 将字符转换成unicode 16进制的字符串形式
     * @param c 字符
     * @return unicode 16进制的字符串
     * @author kane
     * @date 2020-09-07 13:32
     */
    public static String convertUnicode(char c) {
        StringBuilder sb = new StringBuilder(10);
        sb.append("\\u");
        // 取出高8位
        int j = (c >>> 8);
        // 以十六进制的无符号整数形式返回一个整数参数的字符串表示形式
        String tmp = Integer.toHexString(j);
        if (tmp.length() == 1) {
            sb.append("0");
        }
        sb.append(tmp);
        // 取出低8位
        j = (c & 0xFF);
        tmp = Integer.toHexString(j);
        if (tmp.length() == 1) {
            sb.append("0");
        }
        sb.append(tmp);
        return sb.toString();

    }

    /**
     * 将字符串转换成unicode 16进制的字符串形式
     * @param str 输入字符串
     * @return unicode 16进制的字符串
     * @author kane
     * @date 2020-09-07 13:38
     */
    public static String convertUnicode(String str) {
        str = (str == null ? "" : str);
        String tmp;
        StringBuilder sb = new StringBuilder(1000);
        char c;
        int i, j;
        sb.setLength(0);
        for (i = 0; i < str.length(); i++) {
            c = str.charAt(i);
            sb.append("\\u");
            // 取出高8位
            j = (c >>> 8);
            tmp = Integer.toHexString(j);
            if (tmp.length() == 1) {
                sb.append("0");
            }
            sb.append(tmp);
            // 取出低8位
            j = (c & 0xFF);
            tmp = Integer.toHexString(j);
            if (tmp.length() == 1) {
                sb.append("0");
            }
            sb.append(tmp);

        }
        return sb.toString();
    }
    // 乱码判断
    // 1 判断是否有超过中文的unicode编码范围内的特殊字符
    //   1.1 没有则判断是中文，完成
    //   1.2 有超过中文的unicode编码字符，执行后续步骤
    // 2 去掉包含的一些非中英文的特殊标点、换行等符号，如加减乘除等符号，这些符号一般会用到需要保留，不作为乱码标志
    // 3 去掉包含表情符号，这些也不作为乱码标志
    // 4 处理一些中文常用简体被错误转换成繁体或是很少用的中文，这类也算是乱码
    // 5 计算乱码字符长度除以总字符长度，得出乱码率（短文本少数乱码效果会差）

    /**
     * 求Map<K,V>中Key(键)的最大值
     *
     * @param map
     * @return
     */
    public static Object getMaxValue(Map<String, Object> map) {
        if (map == null) {
            return null;
        }
        Set<String> set = map.keySet();
        Object[] obj = set.toArray();
        Arrays.sort(obj);
        return map.get(obj[obj.length - 1]);
    }

    /**
     * 求Map<K,V>中Key(键)的最大值
     *
     * @param map
     * @return
     */
    public static String getMaxKey(Map<String, Double> map) {
        if (map == null) {
            return null;
        }
        Set<String> set = map.keySet();
        Object[] obj = set.toArray();
        Arrays.sort(obj);
        return String.valueOf(obj[obj.length - 1]);
    }

}
